! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
module graph_mod


    use dal
    use errorHandling

    use omutility_mod
    use omnumerics_mod 

    use omallocation_mod

   

  




interface Histogram
    module procedure Histogram1, Histogram2, &
                     Histogram3, Histogram4, &
                     Histogram6, Histogram7
end interface Histogram







real(single), private :: dy = 2.0

real(single), private :: paperWidth = 9.0, paperHeight = 7.0



integer(int8), private :: maxNumBins = 20
integer, private :: labelWidth = 4
integer, private :: axesWidth = 3
integer, private :: histogramWidth = 3
real(single), private :: symbolSize = 1.5
real(single), private :: yLabelOffset = 3.0

integer, private :: white1, black, red, darkred, blue, green, cyan, orange, yellow, magenta
integer, private :: indigo, violet, gray, darkgray, brightgray, darkblue, mediumblue, skyblue
integer, private :: darkgreen, darkyellow, darkcyan, darkmagenta, electricIndigo, electricViolet
integer, private :: histogramColour 


integer, private :: solid = 1
integer, private :: histogramstyle = 1
!**************************************************************
!
!*************************************************************




!integer :: symbol(10) = (/9, 2, 3, 4, 5, 6, 7, 8, 9, 10/)

integer :: symbol(10) = (/5, 3, 2, 4, 7, 8, 8, 9, 0, 1/)

!integer :: ci(10) = (/1, 10, 6, 4, 8, 3, 7, 8, 9, 10/)

integer :: ci(20) = (/1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20/)


type action

    real(single) :: x1, x2, y1, y2
    integer :: width, colour, style
    real(single) :: height
    character(100) :: type

end type action

contains

integer function getRed()
    getRed = red
end function getRed

integer function getBlack()
    getBlack = black
end function getBlack

integer function getmediumBlue()
    getMediumBlue = mediumBlue
end function getMediumBlue






!*************************************************
!
!
!************************************************
subroutine setHistogramStyle(style)
    integer, intent(in) :: style

    histogramStyle = style

end subroutine setHistogramStyle

!*************************************************
!
!
!************************************************
subroutine setHistogramColour(colour)
    integer, intent(in) :: colour

    histogramColour = colour

end subroutine setHistogramColour




!**********************************************************
!
!
!**********************************************************
subroutine setPaperSize(width, height)
    real(single), intent(in) :: width, height

    paperWidth = width
    paperHeight = height

end subroutine setPaperSize








!*****************************************************
!
! Subroutine Histogram
!
!*****************************************************
subroutine Histogram1(ndata, data, noBins, &
                 xLabel, title, x1, x2, y1, y2, colour)
    integer, intent(in) :: ndata
    real(single), intent(in), dimension(ndata) :: data
    integer(int32), intent(in) :: noBins
    character(*), intent(in) :: title, xLabel
    real, intent(in) :: x1, x2, y1, y2
    integer, intent(in), optional :: colour
   
    real(single) ::  xmin, xmax, charSize, yMax, binWidth     
    integer i, colour1
    integer(int32), pointer:: frequency(:) 
   
    character(100) :: yLabel 

    if(present(colour)) then
        colour1 = colour
    else
        colour1 = red
    end if

    charSize = (x2 - x1) * 1.5

    xmin = data(ndata)
    xmax = data(1)
    do i = 1, ndata
        xmin = min(xmin, data(i))
        xmax = max(xmax, data(i))
    end do


    if (abs(xmin - xmax) .lt. 1e-6) then
        call warning("Unable to draw histogram", "xmin=xmax")
        return
    end if


   
     



    

    !******************************
    ! Find the maximum bin size
    !******************************

    frequency => AllocateInt32(1, noBins, "frequency")
    frequency = 0
    binWidth = (xmax - xmin) / real(noBins)  



!    if( xmin .lt. 0.0 .and. xmax .gt. 0.0) then   
!        noLeftBins = int(xmin / binWidth + 0.00000000001);
!        xmin = dble(noLeftBins) * binWidth - binWidth / 2.0;
!        noRightBins = int(xmax / binWidth + 0.00000000001);
!        xmax = dble(noRightBins) * binWidth + binWidth / 2.0;
!    end if



    call computeFrequencies(nData, data, xmin, binWidth, noBins, frequency)
!    do i = 1, ndata
!        k = int((data(i) - xmin) / binWidth + 0.00000001)       
!        k = max(1, k)
!        k = min(noBins, k)
!        frequency(k) = frequency(k) + 1       
!    end do
    
    yMax = real( maxval(frequency) ) * 100.0 / real(ndata) + 10.0
 
    ymax = ceiling(ymax / 10.0) * 10.0

    ymax = ymax * real(ndata) / 100.0

    ymax = min(real(ndata), yMax)

    deallocate(frequency)  



 

     
   


!write(*, *) "Histogram1 line 305"
!    write(*, *) "xmin=", xmin, " xmax=", xmax, " ymax=", ymax


   
     


  
    

    if(noBins > 20) then
       binwidth = (xmax - xmin) / 20.0
 

    else
    end if

  






    
    write(yLabel, "(a, i0, a)") "FREQUENCY (%) (N=", ndata, ")"



 
  

end subroutine Histogram1



  
  

!*****************************************************
! Subroutine Histogram
!
!*****************************************************
subroutine Histogram2(ndata, data, binWidth, &
                 xLabel, title, x1, x2, y1, y2, colour)
    integer, intent(in) :: ndata
    real(single), intent(in), dimension(ndata) :: data
    real(single), intent(inout) :: binWidth
    character(*), intent(in) :: title, xLabel
    real, intent(in) :: x1, x2, y1, y2
   
    integer, intent(in), optional :: colour
 
    real(single) :: xmin, xmax        
    integer :: i, noBins, colour1


    
  
    if(present(colour)) then
        colour1 = colour
    else
        colour1 = red
    end if

 

    xmin = minval(data)
    xmax = maxval(data)


    if (abs(xmin - xmax) .lt. 1e-6) then
        call warning("Unable to draw histogram", "xmin=xmax")
        return
    end if

 

    xmin = 1e12
    xmax = -xmin
    do i = 1, ndata
        xmin = min(xmin, data(i))
        xmax = max(xmax, data(i))
    end do




    xmin = real(floor(xmin))
    xmax = real(ceiling(xmax))


    noBins = int( (xmax - xmin) / binWidth + 0.000001)






!write(*, *) "Histogram2 line 415 xmin=", xmin, " xmax=", xmax

!write(*, *) "nobins=", noBins


    
        
    call Histogram3(ndata, data, noBins, &
                 xLabel, title, x1, x2, y1, y2, &
                 xmin, xmax, colour1)
   
    
    
      

  

 
end subroutine Histogram2


!*****************************************************
! Subroutine Histogram
!
!*****************************************************
subroutine Histogram3(ndata, data, noBins, &
                 xLabel, title, x1, x2, y1, y2, &
                 xmin1, xmax1, colour)
    integer, intent(in) :: ndata
    real(single), intent(in), dimension(ndata) :: data
    integer(int32), intent(in) :: noBins
    character(*), intent(in) :: title, xLabel
    real, intent(in) :: x1, x2, y1, y2
    real(single), intent(in) ::  xmin1, xmax1
  
    integer, intent(in), optional :: colour
   
    real(single) :: charSize, yMax, binWidth, xmin, xmax     
    real(single) :: binWidth1
    integer noBins1, colour1
    integer(int32), pointer:: frequency(:) 
       
    integer :: k = 1
    character(100) :: yLabel 
   
  

    if(present(colour)) then
        colour1 = colour
    else
        colour1 = red
    end if

 
 

    xmin = xmin1
    xmax = xmax1
    if(noBins .lt. 2) return 
 
   
    if (abs(xmin - xmax) .lt. 1e-6) then
        call warning("Unable to draw histogram", "xmin=xmax")
        return
    end if

    charSize = (x2 - x1) * 1.5

   
    !******************************
    ! Find the maximum bin size
    !******************************

    binWidth = (dble(xmax) - dble(xmin)) / dble(noBins)  

    noBins1 = noBins

  


!    if( xmin .lt. 0.0 .and. xmax .gt. 0.0) then
    
!write(*, *) "histogram3 input xmin=", xmin1, " xmax=", xmax1, " binwidth=", binwidth
!        noLeftBins = int( dble(xmin) / dble(binWidth) + 0.00001)
!        xmin = dble(noLeftBins) * dble(binWidth) - dble(binWidth) / 2.0_double
!        noRightBins = int(dble(xmax) / dble(binWidth) + 0.00001)
!        xmax = dble(noRightBins) * dble(binWidth) + dble(binWidth) / 2.0_double
!        noBins1 = abs(noLeftBins) + noRightBins + 1

!write(*, *) "histogram3 Graph line 2958 xmin=", xmin, " xmax=", xmax, " binwidth=", binwidth, &
!" nobins=", noBins, " nobins1=", noBins1
!    end if




    frequency => AllocateInt32(1, noBins1, "frequency")
    frequency = 0


  






    call computeFrequencies(nData, data, xmin, binWidth, noBins, frequency)
  
!    do i = 1, ndata
!        k = int((data(i) - xmin) / binWidth + 0.00000001)       
!        k = max(1, k)
!        k = min(noBins1, k)
!        frequency(k) = frequency(k) + 1           
!    end do





    yMax = min(100, ceiling( 120.0 * real(maxval(frequency)) / real(nData) ))
    yMax = ymax * real(nData) / 100.0 

    ymax = min(real(ndata), yMax)

  
     

  
    
    !***********************************************************************
    ! NB the number of bins cannot exceed 200
    !
    !**********************************************************************
    noBins1 = noBins        
    if(noBins > 200) then
        do            
            noBins1 = noBins1 / 2
            if(noBins1 .le. 200) then
                exit 
            end if
        end do 
    end if
      
   


    deallocate(frequency)  
    nullify(frequency)  
  



   






    !********************************************************************
    !
    ! Draw y axis as a percentage of total
    !
    !*******************************************************************






  
    


  
  

       
     if(noBins > maxNumBins) then
         k = 2
         do 

             binwidth1 = binwidth * dble(k)
             noBins1 = ceiling( (xmax - xmin) / binWidth1 )



    
             if(noBins1 .le. maxNumBins .or. k > 20) then
                 exit
             end if
             k = k * 2
          end do
      
  !       binWidth1 = (xmax1 - xmin1) / dble(maxNumBins);
  !       binWidth1 = ceiling(binWidth1 * 100.0) / 100.0
  !     write(*, *) "Line 589 xmin1=", xmin1, " xmax1=", xmax1, " nobins1=", nobins1, &
  !                 " binwithd1=", binwidth1
  !     noBins1 = maxNumBins + 1

      
    
    else
    end if




    !**********************************************************
    ! Draw the labels
    !**********************************************************

    write(yLabel, "(a, i0, a)") "FREQUENCY (%) (N=", ndata, ")"
    
  




    
end subroutine Histogram3

!*****************************************************
! Subroutine Histogram
!
!*****************************************************
subroutine Histogram4(ndata, data, binWidth, &
                 xLabel, title, x1, x2, y1, y2, &
                 xmin, xmax, colour)
    integer, intent(in) :: ndata
    real(single), intent(in), dimension(ndata) :: data
    real(single), intent(inout) :: binWidth
    character(*), intent(in) :: title, xLabel
    real, intent(in) :: x1, x2, y1, y2
  
    integer, intent(in), optional :: colour
 
    real(single), intent(in) ::  xmin, xmax
    integer :: noBins, colour1  

    if(present(colour)) then
        colour1 = colour
    else
        colour1 = red
    end if

 

    noBins = int( (xmax - xmin) / binWidth + 0.0001)

   



    
    call Histogram3(ndata, data, noBins, &
                 xLabel, title, x1, x2, y1, y2, &
                 xmin, xmax, colour1)
     


 

end subroutine Histogram4






!*****************************************************
!
! Subroutine Histogram
! For drawing two histograms
! on one plot
!
!*****************************************************
subroutine Histogram6(ndata1, data1, caption1, colour1, style1, &
                      ndata2, data2, caption2, colour2, style2, &     
                      binWidth, &
                 xLabel, title, x1, x2, y1, y2, &
                 xmin1, xmax1)
  
    integer(int32), intent(in) :: ndata1, ndata2
    real(single), intent(in), dimension(:) :: data1, data2
    real(single), intent(inout) :: binWidth
    integer, intent(in), optional :: colour1, colour2, style1, style2
    character(*), intent(in) :: title, xLabel
    character(*), intent(in) :: caption1, caption2
   

    real, intent(in) :: x1, x2, y1, y2
    real(single), intent(in) ::  xmin1, xmax1
 
    real(single) :: charSize, yMax, ymax1, ymax2, xmin, xmax     
    real(single) :: xl, xu, xl1, xu1, binwidth1
    integer i, j, noBins
    integer(int32), pointer:: frequency(:) 
    character(200) :: text  
    integer :: k = 1, col1, col2, k1, k2, sty1, sty2, noBins1
 
    
    if(present(colour1)) then
        col1 = colour1
    else
        col1 = red
    end if

    if(present(colour2)) then
        col2 = colour2
    else
        col2 = blue
    end if


    if(present(style1)) then
        sty2 = style1
    else
        sty1 = solid
    end if



    if(present(style2)) then
        sty2 = style2
    else
        sty2 = histogramStyle
    end if

 




    xmin = xmin1
    xmax = xmax1

    if (abs(xmin - xmax) .lt. 1e-6) then
        call warning("Unable to draw histogram", "xmin=xmax")
        return
    end if


!    if( xmin .lt. 0.0 .and. xmax .gt. 0.0) then    
!        noLeftBins = int(xmin / binWidth + 0.00000000001);
!        xmin = dble(noLeftBins) * binWidth - binWidth / 2.0;
!        noRightBins = int(xmax / binWidth + 0.00000000001);
!        xmax = dble(noRightBins) * binWidth + binWidth / 2.0;
!    end if





    charSize = (x2 - x1) * 1.5

   
 
    
    noBins = int(ceiling((real(ceiling(xmax)) - xmin) / binWidth))
    if (noBins < 3) then
        call warning("omgraph", "No of bins < 3")
        return
    end if
            
    

    !******************************
    !
    ! Find the maximum bin size
    !
    !******************************
    frequency => AllocateInt32(1, noBins, "frequency")
    frequency = 0
    yMax1 = 0.0
    yMax2 = 0.0
    frequency = 0



    call computeFrequencies(nData1, data1, xmin, binWidth, noBins, frequency)


        
!    do i = 1, ndata1
!        k = int((data1(i) - xmin) / binWidth + 0.00000001) + 1       
!        k = max(1, k)
!        k = min(noBins, k)
!        frequency(k) = frequency(k) + 1
!    end do

    !*************************************************************
    !
    ! NB Convert to percentages
    !*************************************************************


    yMax1 = min(100, ceiling( 120.0 * real(maxval(frequency)) / real(nData1) ))
  

  
    frequency = 0
    call computeFrequencies(nData2, data2, xmin, binWidth, noBins, frequency, .false.)

!    do i = 1, ndata2
!        k = int((data2(i) - xmin) / binWidth + 0.00000001) + 1       
!        k = max(1, k)
!        k = min(noBins, k)
!        frequency(k) = frequency(k) + 1
!    end do


    yMax2 = min(100, ceiling( 120.0 * real(maxval(frequency)) / real(nData2) ))
   

 
    !*************************************************************
    !
    !
    !
    !*************************************************************
    deallocate(frequency)  
     

    ymax = max(yMax1, yMax2)
   
     
      


    !**************************************************************************
    !
    ! Draw the bins simultaneously
    !
    !**************************************************************************
    

    do i = 1, noBins ! Loop through the bins
        xl = xMin + real(i - 1) * binWidth
        xu = xMin + real(i) * binWidth
          
        k1 = 0  ! The number in each bin for data1
        k2 = 0  ! The number in each bin for data2

        if(i .eq. 1) then
            do j = 1, ndata1
                if( data1(j) .ge. xl .and. data1(j) .le. xu) then 
                    k1 = k1 + 1
                end if
            end do
        else
            do j = 1, ndata1
                if( data1(j) .gt. xl .and. data1(j) .le. xu) then 
                    k1 = k1 + 1
                end if
            end do
        end if 
     
        if(i .eq. 1) then

            do j = 1, ndata2
                if( data2(j) .ge. xl .and. data2(j) .le. xu) then 
                    k2 = k2 + 1
                end if
            end do
        else
            do j = 1, ndata2
                if( data2(j) .gt. xl .and. data2(j) .le. xu) then 
                    k2 = k2 + 1
                end if
            end do
        end if 
     


        !**********************************************************
        !
        !
        !**********************************************************
!        xl = xl + binWidth * 0.1
!        xu = xu - binWidth * 0.1
   
!        if(k1 <= k2) then ! 
!        else
!        end if

            xl1 = xl + 0.25 * binWidth
            xu1 = xu 
            

            xl1 = xl
            xu1 = xl + 0.25 * binWidth
 
            if(k2 > k1) then
                xl1 = xu1
                xu1 = xl + 0.75 * binWidth
            end if
        
    end do









   
    


       
     if(noBins > maxNumBins) then
         k = 2
         do 
             binwidth1 = binwidth * dble(k)
             noBins1 = ceiling( (xmax - xmin) / binWidth1 )
             
             if(noBins1 .le. maxNumBins .or. k > 20) then
                 exit
             end if
             k = k * 2
          end do 
  

    else
    end if




       
 




  
    write(text, '(a, i0, a)') " (N=", ndata1, ")"
 
    write(text, '(a, i0, a)') " (N=", ndata2, ")"
 






   







    
end subroutine Histogram6







!*****************************************************
! Subroutine Histogram
! For drawing several histograms
!
!*****************************************************
subroutine Histogram61(nHists, ndata, data, binWidth, &
                 xLabel, title, caption, x1, x2, y1, y2, &
                 xmin1, xmax1, colour)
    integer(int32), intent(in) :: nHists
    integer(int32), intent(in) :: ndata(:)
    real(single), intent(in), dimension(:, :) :: data
    real(single), intent(inout) :: binWidth
    character(*), intent(in) :: title, xLabel
    character(*), intent(in), optional :: caption(:)

    real, intent(in) :: x1, x2, y1, y2
    real(single), intent(in) ::  xmin1, xmax1
    integer, intent(in), optional :: colour(:)
    real(single) :: charSize, yMax, xmin, xmax     
    real(single), pointer :: tempData(:) => null()        
    integer i, j, noBins
    integer(int32), pointer:: frequency(:) 
  
    integer :: k = 1, colour1(10)
   
   

    if(present(colour)) then
        colour1(1 : nHists) = colour(1 : nHists)
    else
        colour1(1 : 10) = ci(1 : 10)
    
    end if

 

!write(*, *) "Histogram6 line 790"
    xmin = xmin1
    xmax = xmax1

    if (abs(xmin - xmax) .lt. 1e-6) then
        call warning("Unable to draw histogram", "xmin=xmax")
        return
    end if





!    if( xmin .lt. 0.0 .and. xmax .gt. 0.0) then    
!        noLeftBins = int(xmin / binWidth + 0.00000000001);
!        xmin = dble(noLeftBins) * binWidth - binWidth / 2.0;
!        noRightBins = int(xmax / binWidth + 0.00000000001);
!        xmax = dble(noRightBins) * binWidth + binWidth / 2.0;
!    end if





    charSize = (x2 - x1) * 1.5

   
 

!write(*, *) "Histogram6 line 804"


    
    noBins = int(ceiling((real(ceiling(xmax)) - xmin) / binWidth))
    if (noBins < 3) then
        call warning("omgraph", "No of bins < 3")
        return
    end if
            
    
    !******************************
    !
    ! Find the maximum bin size
    !
    !******************************



!write(*, *) "Histogram6 line 822"



    frequency => AllocateInt32(1, noBins, "frequency")
    frequency = 0
    yMax = 0.0
    do j = 1, nHists    
        frequency = 0
        
        do i = 1, ndata(j)
            k = int((data(i, j) - xmin) / binWidth + 0.00000001) + 1       
            k = max(1, k)
            k = min(noBins, k)
            frequency(k) = frequency(k) + 1
        end do
        yMax = max(yMax, real(int(real(maxval(frequency)) * 1.25))) 

    end do
      
    deallocate(frequency)  
     




   
     
      

  
    do i = 1, nHists
 
        tempData => allocateSingle(1, ndata(i), "tempData")
        tempData(1 : nData(i)) = data(1 : nData(i), i)

       


        if (i > 100) then
           

        end if

        deallocate(tempData)
        if (present(caption)) then
 
   
        end if
    end do




   
    if(noBins > 20) then
       binwidth = (xmax - xmin) / 20.0

    else
    end if



!    write(text, "(a)") "NO", ndata(1), ")"
       
 



  

    
end subroutine Histogram61
















!*****************************************************
! Subroutine Histogram
!
!*****************************************************
subroutine Histogram7(noBins, binwidth, xBin, frequency,   &
                 xLabel, title, x1, x2, y1, y2, &
                 xmin, xmax)
   
    integer, intent(in) :: noBins
    real(single), intent(inout) :: binWidth
    real(single) :: xBin(:)
    integer(int32) :: frequency(:)
    
    character(*), intent(in) :: title, xLabel
    real, intent(in) :: x1, x2, y1, y2
    real(single), intent(in) ::  xmin, xmax
    real(single) :: charSize, yMax    
    real(single), pointer :: yBin(:)=> null()
    real(single) :: xl, xu, dr
    integer i, nData
    
    
    character(100) :: yLabel 
 
  
    if(noBins .lt. 2) return 


    nData = sum(frequency(1 : noBins) )


 
  
    if (abs(xmin - xmax) .lt. 1e-6) then
        call warning("Unable to draw histogram", "xmin=xmax")
        return
    end if

    charSize = (x2 - x1) * 1.5

   
    !******************************
    ! Find the maximum bin size
    !******************************

  


    yMax = min(100, ceiling( 150.0 * real(maxval(frequency(1 : noBins))) / real(nData) ))
    yMax = ymax * real(nData) / 100.0 



    ymax = min(real(ndata), yMax)

   
     
    


     
    
  

  
    yBin => allocateSingle(1, noBins, "yBin")
    yBin = real(frequency(1 : noBins))


!write(*, *) "CALLING PGBIN - noBins =", noBins, " xmin=", xmin, " xmax=", xmax
!write(*, *) "THE BINS"


  
    dr = binWidth / 2.0



    if(noBins > 50) then
   

    else        
        do i = 1, noBins



            xl = xbin(i) - dr 
            xu = xbin(i) + dr
       
            if(i .eq. 1) then
                xl = max(xmin, xl)
                xu = min(xbin(i + 1) - dr, xu)
            else if(i .eq. noBins) then            
                xu = min(xmax, xu)
            else
                xu = min(xbin(i + 1) - dr, xu)


            end if   
       




        
        end do
    end if  















    deallocate(yBin);

    !********************************************************************
    ! Draw y axis as a percentage of total
    !*******************************************************************

  
    

    
    if(noBins > 20) then
        binwidth = (xmax - xmin) / 20.0

    else
    end if

    write(yLabel, "(a, i0, a)") "FREQUENCY (%) (N=", ndata, ")"
       



  

    
end subroutine Histogram7








!****************************************
! SetUpColourIndice
! Colour indices used to plot fits regions
!***************************************/
subroutine SetUpDefaultColourIndices( )


   

   

    white1 = 0



    black = 1

    blue = 2



    red = 3

   



    green = 4

    cyan = 5

    orange = 6

    magenta = 7

    darkred = 8

    mediumblue = 9

    darkblue = 10

    skyblue = 11

    gray = 12

    darkgray = 13



    yellow = 14

   
    histogramColour = darkBlue


    brightgray = 15
    darkyellow = 16
    darkgreen = 17

    darkcyan = 18

    darkmagenta = 19 
    electricIndigo = 20
    electricViolet = 21 
  

    violet = 22 
  
    indigo = 23 
  







end subroutine SetUpDefaultColourIndices




















!**************************************************************************
!
!
!
!**************************************************************************
subroutine computeFrequencies(nData, data, xmin, binWidth, noBins, frequency, refineBinSize)
    integer, intent(in) :: nData
    real(single) :: data(:)
    real(single), intent(in) :: xMin
    real(single), intent(inout) :: binWidth
    integer, intent(in) :: noBins
    integer :: frequency(:)
    logical, intent(in), optional :: refineBinSize
    integer :: i, j, k, maxFrequency
  
    do j = 1, 5
        frequency = 0

        do i = 1, ndata
            k = int((data(i) - xmin) / binWidth + 0.00000001)       
            k = max(1, k)
            k = min(noBins, k)
            frequency(k) = frequency(k) + 1       
        end do
        if( .not. present(refineBinSize)) then
            exit
        end if
        
        maxFrequency = maxVal(frequency) 
        if(real(maxFrequency) / real(ndata) >= 0.1) then
            exit 
        else
         
            binWidth = binWidth / 2.0
        end if
    end do

end subroutine computeFrequencies

!************************************************
!
!
!***********************************************

!**************************************************
!
!
!************************************************
subroutine drawLine(nData, xData, yData)
    integer, intent(in) :: ndata
    real(single), dimension(:) :: xData, yData
    integer :: j
    do j = 2, nData
    end do

end subroutine drawLine




end module graph_mod
